@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2017, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2020, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -167,13 +167,13 @@ AcpiUtInsertDigit (
 static ACPI_STATUS
 AcpiUtStrtoulMultiply64 (
     UINT64                  Multiplicand,
-    UINT64                  Multiplier,
+    UINT32                  Base,
     UINT64                  *OutProduct);
 
 static ACPI_STATUS
 AcpiUtStrtoulAdd64 (
     UINT64                  Addend1,
-    UINT64                  Addend2,
+    UINT32                  Digit,
     UINT64                  *OutSum);
 
 
@@ -207,10 +207,16 @@ AcpiUtConvertOctalString (
 
     while (*String)
     {
-        /* Character must be ASCII 0-7, otherwise terminate with no error */
-
+        /*
+         * Character must be ASCII 0-7, otherwise:
+         * 1) Runtime: terminate with no error, per the ACPI spec
+         * 2) Compiler: return an error
+         */
         if (!(ACPI_IS_OCTAL_DIGIT (*String)))
         {
+#ifdef ACPI_ASL_COMPILER
+            Status = AE_BAD_OCTAL_CONSTANT;
+#endif
             break;
         }
 
@@ -263,10 +269,16 @@ AcpiUtConvertDecimalString (
 
     while (*String)
     {
-        /* Character must be ASCII 0-9, otherwise terminate with no error */
-
+        /*
+         * Character must be ASCII 0-9, otherwise:
+         * 1) Runtime: terminate with no error, per the ACPI spec
+         * 2) Compiler: return an error
+         */
         if (!isdigit (*String))
         {
+#ifdef ACPI_ASL_COMPILER
+            Status = AE_BAD_DECIMAL_CONSTANT;
+#endif
            break;
         }
 
@@ -319,10 +331,16 @@ AcpiUtConvertHexString (
 
     while (*String)
     {
-        /* Must be ASCII A-F, a-f, or 0-9, otherwise terminate with no error */
-
+        /*
+         * Character must be ASCII A-F, a-f, or 0-9, otherwise:
+         * 1) Runtime: terminate with no error, per the ACPI spec
+         * 2) Compiler: return an error
+         */
         if (!isxdigit (*String))
         {
+#ifdef ACPI_ASL_COMPILER
+            Status = AE_BAD_HEX_CONSTANT;
+#endif
             break;
         }
 
@@ -419,15 +437,39 @@ BOOLEAN
 AcpiUtDetectHexPrefix (
     char                    **String)
 {
+    char                    *InitialPosition = *String;
 
+    AcpiUtRemoveHexPrefix (String);
+    if (*String != InitialPosition)
+    {
+        return (TRUE); /* String is past leading 0x */
+    }
+
+    return (FALSE);     /* Not a hex string */
+}
+
+
+/*******************************************************************************
+ *
+ * FUNCTION:    AcpiUtRemoveHexPrefix
+ *
+ * PARAMETERS:  String                  - Pointer to input ASCII string
+ *
+ * RETURN:      none
+ *
+ * DESCRIPTION: Remove a hex "0x" prefix
+ *
+ ******************************************************************************/
+
+void
+AcpiUtRemoveHexPrefix (
+    char                    **String)
+{
     if ((**String == ACPI_ASCII_ZERO) &&
         (tolower ((int) *(*String + 1)) == 'x'))
     {
         *String += 2;        /* Go past the leading 0x */
-        return (TRUE);
     }
-
-    return (FALSE);     /* Not a hex string */
 }
 
 
@@ -518,7 +560,7 @@ AcpiUtInsertDigit (
  * FUNCTION:    AcpiUtStrtoulMultiply64
  *
  * PARAMETERS:  Multiplicand            - Current accumulated converted integer
- *              Multiplier              - Base/Radix
+ *              Base                    - Base/Radix
  *              OutProduct              - Where the product is returned
  *
  * RETURN:      Status and 64-bit product
@@ -532,28 +574,36 @@ AcpiUtInsertDigit (
 static ACPI_STATUS
 AcpiUtStrtoulMultiply64 (
     UINT64                  Multiplicand,
-    UINT64                  Multiplier,
+    UINT32                  Base,
     UINT64                  *OutProduct)
 {
     UINT64                  Product;
+    UINT64                  Quotient;
 
 
     /* Exit if either operand is zero */
 
     *OutProduct = 0;
-    if (!Multiplicand || !Multiplier)
+    if (!Multiplicand || !Base)
     {
         return (AE_OK);
     }
 
-    /* Check for 64-bit overflow before the actual multiplication */
-
-    if (Multiplicand > (ACPI_UINT64_MAX / Multiplier))
+    /*
+     * Check for 64-bit overflow before the actual multiplication.
+     *
+     * Notes: 64-bit division is often not supported on 32-bit platforms
+     * (it requires a library function), Therefore ACPICA has a local
+     * 64-bit divide function. Also, Multiplier is currently only used
+     * as the radix (8/10/16), to the 64/32 divide will always work.
+     */
+    AcpiUtShortDivide (ACPI_UINT64_MAX, Base, &Quotient, NULL);
+    if (Multiplicand > Quotient)
     {
         return (AE_NUMERIC_OVERFLOW);
     }
 
-    Product = Multiplicand * Multiplier;
+    Product = Multiplicand * Base;
 
     /* Check for 32-bit overflow if necessary */
 
@@ -572,7 +622,7 @@ AcpiUtStrtoulMultiply64 (
  * FUNCTION:    AcpiUtStrtoulAdd64
  *
  * PARAMETERS:  Addend1                 - Current accumulated converted integer
- *              Addend2                 - New hex value/char
+ *              Digit                   - New hex value/char
  *              OutSum                  - Where sum is returned (Accumulator)
  *
  * RETURN:      Status and 64-bit sum
@@ -586,7 +636,7 @@ AcpiUtStrtoulMultiply64 (
 static ACPI_STATUS
 AcpiUtStrtoulAdd64 (
     UINT64                  Addend1,
-    UINT64                  Addend2,
+    UINT32                  Digit,
     UINT64                  *OutSum)
 {
     UINT64                  Sum;
@@ -594,12 +644,12 @@ AcpiUtStrtoulAdd64 (
 
     /* Check for 64-bit overflow before the actual addition */
 
-    if ((Addend1 > 0) && (Addend2 > (ACPI_UINT64_MAX - Addend1)))
+    if ((Addend1 > 0) && (Digit > (ACPI_UINT64_MAX - Addend1)))
     {
         return (AE_NUMERIC_OVERFLOW);
     }
 
-    Sum = Addend1 + Addend2;
+    Sum = Addend1 + Digit;
 
     /* Check for 32-bit overflow if necessary */
 
